/**************************************************************

   Program Title:  Wasp AVR Monitor  Data Logger
   
   Program Name:  Wasp_Monitor.c	  Version: 0.91
   
   Author:  SS	  					  Date:  June 22, 2009
   
   Compiled with ImageCraft ICCAVR Standard V7.13A.
   
   This verison is designed for the ATmega644 processor:
   		64K Flash, 2K EEPROM, 4K RAM
		
   If the Serial Flash and/or Accelerometer are installed then a 10MHz
   crystal should be installed in place of the 20MHz crystal as the 20MHz
   crystal requires the processor be powered by 5V which the serial flash
   and Accelerometer can't handle.
		
   Fuse bit:  FF D9 E2  - Disable JTAG - use internal OSC
   Fuse bit:  FF D9 EE  - Disable JTAG - use external 10MHz Xtal
   Fuse bit:  FF D9 E7  - Disable JTAG - use external 20MHz Xtal
	
   This program will not run properly if DIV8 fuse bit is set (programmed)
   and JTAG pins are enabled - this is the chip default.
		
   The Wasp Monitor is a set of functions that configure Wasp AVR internal
   peripherals for interrupt driven operation.  Custom applications can easily
   be built on top to these basic functions.  Included is a complete data 
   acquistion controller with user preferences loaded into EEPROM.  The Monitor
   communicates with the user via an RS-232 port using the CDP.   The AVR is 
   a smart powerful 8 bit RISC processor that can be easily programmed in C
   with on chip TWI, SPI and analog and digital IO.  Functions are included 
   to read/write data blocks to the on board Serial Flash.  On board sensors
   (if included) include a dual axis 2G accelerometer, temperature and light
   sensor and battery voltage monitor.
   
   Sensors:
   		AD0 - Accelerometer channel Y
		AD1 - Accelerometer channel Z
		AD5 - Accelerometer channel X
		AD2 - Battery Voltage
		AD3 - Temperature 
		AD4 - Light level
		
   Key functions:
   	   SPI Master with read/write function to serial flash
	   TWI 2-Wire Master Interrupt Controller
	   Real Time Clock setup with four configurable software timers
	   Full duplex interrupt drive USART with XON/XOFF flow control
	   1msec timer with four configurable software times
	   Analog controller setup
	   Digital port configuration
	   Accelerometer reader
	   Temperature reader
	   Light level reader
	   IntelHex32 download/upload for EEPROM, AVRFLASH and Serial Flash
	   Auto clock calibration
   
   Copyright 2004-2010, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2010, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROROTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OC MACHINES, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: Wasp_Monitor.c,v $
 * Revision 0.91  2010/03/04 SS
 *    DC200 Support added
 * Revision 0.90  2009/06/20 SS
 *    Serial Flash data logger support for 3-axis rate gyro added
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, Serial Flash writer, XON/XOFF, 9,600baud
 *    real time clock and auto-internal osc calibration.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
#include <iom644v.h>
#include <macros.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include "Wasp_Uart.h"
#include "Wasp_Defines.h"
#include "Wasp_Extern.h"

//  Variable declarations
unsigned int bytesonline,inteladdress;
unsigned char programmode,crc,intelbinary[34];  // length can vary
unsigned char hexbyte[3],hexaddress[5],intelrecordcommand;
unsigned char seconds,minutes,hours,days,rtcclockcnt,loggingmode,data2log;
unsigned char flashtype0,flashtype1,flashpageloaded,flashpageboundarytraversed;
unsigned char flashprogrammingactive,flashdatainbuffer,ascii[16],bufferidptr;
unsigned char version[30],i2cmode,armpd2,smartlcdstate,digitalcompassstate;
unsigned char TWI_slaveAddress,TWI_masterAddress;

unsigned int flashpagesize0,flashpagesize1,oldinteladdress,pagecnt;
unsigned int intelsuperaddress,oldpage,oldbyteoffset,totalrxcnt;
unsigned int rtccountdown1,rtccountdown2,rtccountdown3,rtccountdown4;
unsigned int mseccountdown1,mseccountdown2,mseccountdown3,flashcountdown;

unsigned char messageBuf[TWIMESSAGELENGTH];

unsigned int totalbytecnt,xoncnt,xoffcnt;
unsigned char frameerror,dataoverrun;
int  cursorlocation,defaultlograte;
long flashaddress;

int _textmode;

/* Static Variables */
unsigned char UART_RxBuf[UART_RX_BUFFER_SIZE];
unsigned char UART_RxHead;
unsigned char UART_RxTail;
unsigned char UART_TxBuf[UART_TX_BUFFER_SIZE];
unsigned char UART_TxHead;
unsigned char UART_TxTail;
char  xoffsent, xonsent, enablexonoff, echoinchars,sendxoff;
char  rxcnt,checkflag,rxcntmax;
unsigned char  instrg[8],vlow,vhigh;

unsigned char FlashAddress[3];
const char signon1[] = "\n\r\n\rWasp ATmega644 Monitor Data Logger V0.91\n\r";
const char signon2[] = "(c) Copyright 2004-2010,  SOC Robotics, Inc.\n\r";
const char signon3[] = "\n\r   D[E|A|F]  - Display AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon4[] = "   P[E|A|F]  - Program AVR EEPROM, AVR Flash or Flash memory\n\r";
const char signon5[] = "   L   - Enter log sensor data mode\n\r";
const char signon6[] = "   G   - Measure gravity using accelerometer\n\r";
const char signon62[] = "   D   - Return digital compass reading\n\r";
const char signon65[] = "   Rn  - Measure rate gyro 1,2 or 3 axis\n\r";
const char signon66[] = "   RG  - Measure gravity and 3-axis gyro\n\r";
const char signon67[] = "   M   - Measure all analog inputs and output statistics\n\r";
const char signon7[] = "   V   - Version\n\r";
const char signon8[] = "   C   - Digital Compass heading (deg)\n\r";
const char signon83[] = "   T   - Get/Set Time\n\r";
const char signon85[] = "   A   - Read all Analog channels\n\r";
const char signon9[] = "   S   - Signon message\n\r";

const char displaymessage1[] = "Enter start address and length: ";
const char displaymessage2[] = "\n\rDisplay AVR Flash from 0x0054 to 254\n\r";
const char displaymessage3[] = "\n\rDisplay SPI Flash from 0 to 100\n\r";
const char displaymessage33[] = "\n\rNo mode selected\n\r";

const char progmessage[] = "Wasp Monitor Version: ";

const char displaymessage4[] = "\n\rLoad EEPROM from address ";
const char displaymessage5[] = " length ";
const char displaymessage6[] = "\n\rLoad SPI Flash from address ";

const char loadmessage1[] = "\n\rLoad EEPROM\n\r";
const char loadmessage2[] = "\n\rLoad SPI Flash\n\r";
const char loadmessage3[] = "\n\rLoad AVR Flash\n\r";

// Function declarations
unsigned char EEPROMwrite(int,unsigned char);
unsigned char EEPROMread(int);
int printf(CONST char *, ...);
int sprintf(char *, CONST char *, ...);
int scanf(CONST char *, ...);
int sscanf(char *, CONST char *, ...);

// Initial MMA7260 3-Axis accelerometer
void Init_3Axis_Accelerometer(char mode)
{
 	 // Set direction bits
	 //   SG2 - PC5, SG1 - PC4, SLP - PC3
	 DDRC |= 0x3C;
 	 // Enable Accelerometer
	 SetBit(PORTC,PORTC3);
	 
	 if(mode==G15MODE) { ClrBit(PORTC,PORTC5); ClrBit(PORTC,PORTC5); }
	 else if(mode==G2MODE) { ClrBit(PORTC,PORTC5); SetBit(PORTC,PORTC5); }
	 else if(mode==G4MODE) { SetBit(PORTC,PORTC5); ClrBit(PORTC,PORTC5); }
	 else if(mode==G6MODE) { SetBit(PORTC,PORTC5); SetBit(PORTC,PORTC5); }
}

void Display_Gravity_UART(char mode)
{
    unsigned char firstmeasurement;
	unsigned char inchar;
    int i,xgrav,ygrav,zgrav,xmin,xmax;
	int xgravity,ygravity,zgravity;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
	float xfgrav,yfgrav,zfgrav;
	
	Init_3Axis_Accelerometer(G15MODE);
	if(mode==GONLY) printf("\n\r3-Axis Acceleration\n\r",0x00);

	while(!(inchar=getcharnw())) {
		xgravity = ygravity = zgravity = 0;	
		for(i=0; i<4; i++) {
			xgravity += AnalogCh(5,EXTERNALREF);
			ygravity += AnalogCh(0,EXTERNALREF);
			zgravity += AnalogCh(1,EXTERNALREF);
			}

		xgravity = (xgravity/4);
		ygravity = (ygravity/4);
		zgravity = (zgravity/4);

		printf("X Y Z accel = %4d %4d %4d  \r",xgravity,ygravity,zgravity);
		}
	printf("\n\r-");
}

// Rate gyro change is 0.67mv per degree or +-67mv
//   Default output is 1.350V - 1.35/3.3 = 0.409
//   0.409*999 = 418
void Display_RateGyro_UART(unsigned char numaxis)
{
	unsigned char inchar;
    int i,gyrorate,gyrorate2,gyrorate3,gyromin,gyromax;
	int xgravity,ygravity,zgravity;
	
	if(numaxis=='1')  {
	    printf("\n\rRate Gyro Output - one axis\n\r");
		printf("        rate  min   max   diff\n\r");
	    gyromin = gyromax = gyrorate = AnalogCh(4,EXTERNALREF);
	    gyromin = gyromax = gyrorate = AnalogCh(4,EXTERNALREF);
	
	    while(!(inchar=getcharnw())) {
	        gyrorate = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    if(gyrorate<gyromin) gyromin = gyrorate;
		    if(gyrorate>gyromax) gyromax = gyrorate;

		    printf("gyro = %4d  %4d  %4d  %4d  \r",gyrorate,gyromin,gyromax,gyromax-gyromin);
		    }
		}
	else if(numaxis=='2') {
		printf("\n\rRate Gyro Output - two axis\n\r");
	
	    while(!(inchar=getcharnw())) {
	        gyrorate = gyrorate2 = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    gyrorate2 += AnalogCh(6,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    gyrorate2 = (gyrorate2/4);

		    printf("gyro = %4d  %4d  \r",gyrorate,gyrorate2);
		    }
	    }
	else if(numaxis=='3') {
		printf("\n\rRate Gyro Output - three axis\n\r");
	
	    while(!(inchar=getcharnw())) {
	        gyrorate = gyrorate2 = gyrorate3 = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    gyrorate2 += AnalogCh(6,EXTERNALREF);
			    gyrorate3 += AnalogCh(7,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    gyrorate2 = (gyrorate2/4);
		    gyrorate3 = (gyrorate3/4);

		    printf("gyro = %4d  %4d  %4d  \r",gyrorate,gyrorate2,gyrorate3);
		    }
	    }
	else if(numaxis=='g') {
		printf("\n\r3-Axis Acceleration + 3-Axis Rate Gyro\n\r");
	
		Init_3Axis_Accelerometer(G15MODE);
	    while(!(inchar=getcharnw())) {
			xgravity = ygravity = zgravity = 0;
		    for(i=0; i<4; i++) {
			    xgravity += AnalogCh(5,EXTERNALREF);
			    ygravity += AnalogCh(0,EXTERNALREF);
			    zgravity += AnalogCh(1,EXTERNALREF);
			    }

		    xgravity = (xgravity/4);
		    ygravity = (ygravity/4);
		    zgravity = (zgravity/4);

		    printf("X Y Z accel = %4d  %4d  %4d ",xgravity,ygravity,zgravity);
			
	        gyrorate = gyrorate2 = gyrorate3 = 0;
		    for(i=0; i<4; i++) {
			    gyrorate += AnalogCh(4,EXTERNALREF);
			    gyrorate2 += AnalogCh(6,EXTERNALREF);
			    gyrorate3 += AnalogCh(7,EXTERNALREF);
			    }

		    gyrorate = (gyrorate/4);
		    gyrorate2 = (gyrorate2/4);
		    gyrorate3 = (gyrorate3/4);

		    printf("gyro = %4d  %4d  %4d  \r",gyrorate,gyrorate2,gyrorate3);
					xgravity = ygravity = zgravity = 0;	
		    }
	    }	
		printf("\n\r-");
}

char FlashBlankCheck()
{
 	 char byte;
	 
	 Release_SPI();
	 Init_SPI_Master();
	 FlashChipSelect(LOW);  
	 SPI_StartReadFlashPage(0);
	 byte = SPI_MasterReceive();
	 FlashChipSelect(HIGH); 
	 Release_SPI(); 
	 if(byte=='y') return YES;
     else return NO;
}

// Retreive logged flash data until no more left.
// The retreival prints the time stamp and flash page number
// then outputs three, six or eight parameters per line.
// Data structure is as follows:
//    Data present in page - 'y' else anything else
//    Time stamp - day,hour,min,second,msec 
void RetreiveFlashData()
{
     unsigned char notfinished,msec,second,min,hour,day,loggeddata;
 	 unsigned char i,j,byte,bytesonlastline,asciicnt,crcvalue,tempvalue;
	 unsigned char highbyte,lowbyte;
	 int xaxis,yaxis,zaxis,agyro,bgyro,cgyro,pagecnt,lograte,buflength;
	 int data[8];
	 long recordcnt;
	 float floattime;
	 
	 Release_SPI();
	 Init_SPI_Master();
	 //  Set PC4 low - Flash CS
	 FlashChipSelect(LOW);  
	 SPI_StartReadFlashPage(0);
	 pagecnt = 1;
	 recordcnt = 1;
	 notfinished = YES;
	 while(notfinished==YES) {
	     byte = SPI_MasterReceive();
		 // See if first page byte is 'y' - process page if it is
		 if(byte=='y') {
		     loggeddata = SPI_MasterReceive();
		     // retrieve log rate
			 lograte = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
		     // Retrieve time stamp
			 day = SPI_MasterReceive();
			 hour = SPI_MasterReceive();
			 min = SPI_MasterReceive();
			 second = SPI_MasterReceive();
			 msec = SPI_MasterReceive();
			 if(pagecnt==1)  printf("Page=%4d %c %4d %d:%d:%d:%d.%d\n\r",pagecnt++,loggeddata,lograte,
			                                     day,hour,min,second,msec*4);
			 floattime = (float) second + ((float) (msec*4))/1000.0;
			 
			 if((loggeddata=='a')||(loggeddata=='g')) {		 
			     if(flashtype0==AT45DB081ID) buflength = 41;
	 		     else if(flashtype0==AT45DB642ID) buflength = 169;

			     for(i=0; i<buflength; i++) {
			         xaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         yaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         zaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();	
				     printf("%ld ",recordcnt++);
			         printf("%d:%d:%d:%2.3f",day,hour,min,floattime);
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 printf(" %4d %4d %4d\n\r",xaxis,yaxis,zaxis);
				     }
			     SPI_MasterReceive();			 		 		 
		         }
			 else if(loggeddata=='b') {
			     if(flashtype0==AT45DB081ID) buflength = 20;
	 		     else if(flashtype0==AT45DB642ID) buflength = 84;

			     for(i=0; i<buflength; i++) {
			         xaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         yaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
			         zaxis = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					 agyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					 bgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					 cgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();	
				     printf("%ld ",recordcnt++);
			         printf("%d:%d:%d:%2.3f",day,hour,min,floattime);
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 printf(" %4d %4d %4d ",xaxis,yaxis,zaxis);
					 printf("%4d %4d %4d\n\r",agyro,bgyro,cgyro);
				     }
				 // Skip the blank bytes at the end of the page
				 for(i=0; i<7; i++ )  SPI_MasterReceive();			 		 		 
		         }

			 else if(loggeddata=='8') {
			     if(flashtype0==AT45DB081ID) buflength = 15;
	 		     else if(flashtype0==AT45DB642ID) buflength = 63;

			     for(i=0; i<buflength; i++) {
				     for(j=0; j<8; j++)  {
					     //lowbyte = SPI_MasterReceive();
						 //highbyte = SPI_MasterReceive();
						 //bgyro = highbyte*256 + lowbyte;
					     //if(i==0) printf("%x %x %d\n\r",highbyte,lowbyte,bgyro);
					     bgyro = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();
					     data[j] = bgyro;
						 }					      
				     printf("%ld ",recordcnt++);
			         printf("%d:%d:%d:%2.3f",day,hour,min,floattime);
					 floattime += ((float) lograte)/1000.0;
					 if(floattime>60.0) {
					     floattime -= 60.0;
						 min++;
						 if(min>59) {
						     min = 0;
							 hour++;
							 if(hour>23) {
							     hour = 0;
								 day++;
								 }
							 }
						 }
					 printf(" %4d %4d %4d ",data[0],data[1],data[2]);
					 printf(" %4d %4d %4d ",data[3],data[4],data[5]);
					 printf("%4d %4d\n\r",data[6],data[7]);
				     }
				 // Skip the blank bytes at the end of the page
				 for(i=0; i<7; i++ )  SPI_MasterReceive();			 		 		 
		         }
			  }
		 else {
		     printf("Finished processing pages");
			 notfinished = NO;
			 } 
		 }
     
	 // Set PC4 high - deselect Flash chip
	 FlashChipSelect(HIGH); 
	 Release_SPI(); 
}

void FlashChipErase()
{
 	 Release_SPI();
	 Init_SPI_Master();
	 printf("\n\rErase serial flash chip");
	 ChipErase();
	 Release_SPI();
	 Init_SPI_Master();
	 WaitForFlashFree();
	 Release_SPI();
}

void DisplayData2Log()
{
	 printf("\n\rData logged is ");
	 if(data2log=='a') printf("accelerometer only\n\r");
	 else if(data2log=='r') printf("rate gyro only\n\r");
	 else if(data2log=='b') printf("accelerometer and rate gyro\n\r");
	 else if(data2log=='8') printf("all 8 analog channels\n\r");
	 else printf("unknown selection\n\r");
}	 

// This function manages the real time logging of sample data
// to the serial flash chip, starts/stops the logging and retrieves
// logged data
void LogDataToSerialFlash()
{
 	 char inchar,inchar1,inchar2;
	 
 	 printf("\n\rEntering log data to serial flash mode\n\r");
	 printf("Default log rate is %d msec per sample",defaultlograte);
	 DisplayData2Log();
	 printf("Commands:\n\r");
	 printf("  a - Arm switch/disarm PD2 - PD2\n\r");
	 printf("  e - Erase chip\n\r");
	 printf("  b - Start data logging\n\r");
	 printf("  s - Select data to log\n\r");
	 printf("  h - Halt data logging\n\r");
	 printf("  r - Retrieve data\n\r");
	 printf("  l - log rate (msec)\n\r");
	 printf("  q - Quit data logging mode\n\r>");
	 
	 inchar = getcharr();
	 putchar(inchar);
	 inchar2 = 'g';
	 while((inchar!='q')&&(inchar2!='b'))  {
	     switch(inchar) {
	         case 'a':
			     if(armpd2==YES) {
				     printf("\n\rDisarm PD2");
				     armpd2 = NO;
					 SaveEepromPreferences();
					 }
				 else {
				     printf("\n\rArm PD2");
				     armpd2 = YES;
					 SaveEepromPreferences();
					 }
		         break;
	         case 'e':
			     FlashChipErase();
		         break;
		     case 'b':
			 	 Init_3Axis_Accelerometer(G15MODE);
				 GREEN_LED(ON);
				 if(FlashBlankCheck()==YES) {
				     RED_LED(ON);
				     FlashChipErase();
					 RED_LED(OFF);
					 }
			     loggingmode = START;
				 bufferidptr = 1;
				 pagecnt = 1;
				 inchar2 = 'b';
				 flashaddress = 0;
				 printf("\n\rLogging mode started");
		         break;
			 case 's':
				 printf("\n\rSelect data to log [a-accelerometer, g-gyro, b-both, 8-all analog]: ");
				 data2log = getchar();
				 putchar(data2log);
				 SaveEepromPreferences();
				 DisplayData2Log();
				 break;
		     case 'h':
			     loggingmode = HALT;
				 GREEN_LED(OFF);
				 printf("\n\rLogging mode halted");
		         break;
		     case 'l':
			     printf("\n\rEnter new log rate (msec): ");
			     defaultlograte = readint();
				 SaveEepromPreferences();
				 printf("\n\rLog rate changed to %d msec",defaultlograte);
		         break;
		     case 'r':
			     printf("\n\rRetreive logged data\n\r");
				 // Retreive logged data from each flash page
				 // until no more data left
				 RetreiveFlashData();
		         break;
		     }
		if(inchar2!='b') {
		    printf("\n\r>");
		    inchar = getcharr();
		    putchar(inchar);
			}
		}
}

// Log data scheduler
// Always fill a flash page with data before returning
// This keeps the scheduler very simple
// This version logs the 3-axis accelerometer
void LogData(int logmsec)
{
	 int i,j,buflength;
	 union {
	    unsigned char tempbuf[2];
		int outiint;
		} u;
	 
	 Release_SPI();
	 Init_SPI_Master(); 	
	 
	 // Serial Flash CS high
	 FlashChipSelect(LOW);	
	  
	 //  Select Buffer to write too
	 if(bufferidptr==1)  SPI_MasterTransmit(SF_WRITE_BUFFER1);
	 else  SPI_MasterTransmit(SF_WRITE_BUFFER2);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(0);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
 
	 SPI_MasterTransmit('y');
	 SPI_MasterTransmit(data2log);
	 u.outiint = logmsec;
	 SPI_MasterTransmit(u.tempbuf[1]);
	 SPI_MasterTransmit(u.tempbuf[0]);
	 SPI_MasterTransmit(days);
	 SPI_MasterTransmit(hours); 
	 SPI_MasterTransmit(minutes);
	 SPI_MasterTransmit(seconds);
	 SPI_MasterTransmit(rtcclockcnt);
	 
	 //printf("Flashtype = %x\n\r",flashtype0);
	 
	 if((data2log=='a')||(data2log=='g'))  {
	     if(flashtype0==AT45DB081ID) buflength = 41;
	     else if(flashtype0==AT45DB642ID) buflength = 169;
	 
	     for(i=0; i<buflength; i++) {
		     if(data2log=='a') {
	             // X axis accelerometer
	             u.outiint = AnalogCh(5,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // Y axis accelerometer
	             u.outiint = AnalogCh(0,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // Z axis accelerometer
	             u.outiint = AnalogCh(1,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
	             }
			 else if(data2log=='g') {
	             // A axis gyro
	             u.outiint = AnalogCh(4,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // B axis gyro
	             u.outiint = AnalogCh(6,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
		         // C axis gyro
	             u.outiint = AnalogCh(7,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
			     }
		    // Delay for the sample period
		    Delaymsec(logmsec);
			}
		 }
	 else if(data2log=='b') {
	     if(flashtype0==AT45DB081ID) buflength = 20;
	     else if(flashtype0==AT45DB642ID) buflength = 84;
	 
	     for(i=0; i<buflength; i++) {
	         // X axis accelerometer
	         u.outiint = AnalogCh(5,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // Y axis accelerometer
	         u.outiint = AnalogCh(0,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // Z axis accelerometer
	         u.outiint = AnalogCh(1,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
			 
	         // A axis gyro
	         u.outiint = AnalogCh(4,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // B axis gyro
	         u.outiint = AnalogCh(6,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		     // C axis gyro
	         u.outiint = AnalogCh(7,EXTERNALREF);
	         SPI_MasterTransmit(u.tempbuf[1]);
	         SPI_MasterTransmit(u.tempbuf[0]);
		 
		     // Delay for the sample period
		     Delaymsec(logmsec);
	         }
		 }
	 else if(data2log=='8') {
	     if(flashtype0==AT45DB081ID) buflength = 15;
	     else if(flashtype0==AT45DB642ID) buflength = 63;
	 
	     for(i=0; i<buflength; i++) {
			 for(j=0; j<8; j++)  {
	             u.outiint = AnalogCh(j,EXTERNALREF);
	             SPI_MasterTransmit(u.tempbuf[1]);
	             SPI_MasterTransmit(u.tempbuf[0]);
				 }
			
		     // Delay for the sample period
		     Delaymsec(logmsec);
	         }	
		 }
		 
	 // Serial Flash CS high
	 FlashChipSelect(HIGH);
	 
	 // Store flash buffer to flashpage
	 FlashChipSelect(LOW);
	  
	 // Transfer buffer n to flash memory
	 if(bufferidptr==1) SPI_MasterTransmit(SF_BUF1_TO_FLASH_NOERASE);
	 else SPI_MasterTransmit(SF_BUF2_TO_FLASH_NOERASE);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Progamming the flash can take up to 40msec so make sure the data 
	 // logging rate is not to high
	 FlashChipSelect(HIGH);
	 
	 // Flip the bufferid ptr
	 if(bufferidptr==1) bufferidptr = 2;
	 else bufferidptr = 1;
 
	 //printf("%d ",pagecnt);
	 
 	 if(flashtype0==AT45DB081ID)  {
	     flashaddress += 256;
		 if(++pagecnt>2048) loggingmode = HALT;
		 }
	 else if(flashtype0==AT45DB642ID)  {
	     flashaddress += 1024;
	 	 if(++pagecnt>8192)  loggingmode = HALT;
		 }

	 Release_SPI();
}

// Read all the analog inputs and output min/max status
void MeasureAnalogInputs()
{
    int i,j,min,max,analogvalue,analogtotal;
	
	printf("\n\rAnalog input summary\n\r");
	printf("Ch  average   min   max   diff\n\r");
	
	for(i=0; i<8; i++) {
		analogvalue = AnalogCh(i,EXTERNALREF);
		analogtotal = 0;
		min = max = analogvalue;
	    for(j=0; j<10; j++)  {
		    analogvalue = AnalogCh(i,EXTERNALREF);
			if(analogvalue < min)  min = analogvalue;
			if(analogvalue > max)  max = analogvalue;
			analogtotal += analogvalue;
			}
		printf("%d     %d    %d    %d    %d",i,analogtotal/10,min,max,max-min);
		if(i==0) printf("  Y axis acceleration");
		else if(i==1) printf("  Z axis acceleration");
		else if(i==4) printf("  A Gyro");
		else if(i==5) printf("  X axis acceleration");
		else if(i==6) printf("  B Gyro");
		else if(i==7) printf("  C Gyro");
		printf("\n\r");
		}
}

void DisplayCompass()
{
 	 int digitalcompassheading = 180;
	 
	 Enumerate_DigitalCompass();
	 
	 Delaymsec(300);
	 digitalcompassheading = Read_DigitalCompass_Heading();
 	 printf("\n\rDigital compass heading = %d\n\r",digitalcompassheading);
}


//*************************************************************
//  Main work starts here
//*************************************************************
void main()
{
	unsigned char inchar,inbyte,twistatus1,twistatus2,twistatus3,twistatus4;
	unsigned char inbytes[5],toggle,slaverx,firstmeasurement,numaxis;
	int i,icnt,j,loopsetting,loopmode,blackfinresetmode,countervalue,numbr;
	int loopmode2,num1,num2,xgrav,ygrav,xmin,xmax,iforce,dataoutputperiod;
	int ylocation,centerpressed;
	long xgravity,ygravity,loopcount,ivalue,bvalue,cvalue;
	float gravity,xaccel,yaccel,force,bforce,cforce,dforce,fforce,roll,pitch;
			
	Init_AVR();
	rxcntmax = totalrxcnt = 0;
	
	loopsetting = 500;   // 1/2 second delay
	loopmode = 0;  toggle = 0;
	mseccountdown1 = loopsetting;
	loopcount = 0;
	dataoutputperiod = 10;  // seconds
	//LCD_Switches = 0x00;   // Switch toggle values
	_textmode = 1;
	
	// Insert copyright - version reference in Flash memory
	strcpy(version,"SOC Wasp  R0.91 (c) 2010");

	// Get Serial Flash status
//	CheckFlashTypes();
	
	// Show something is happening
	while(0) {  
		BlinkRed(5);
		BlinkGreen(5);
		}
		
	_SEI(); // Enable interrupts => enable UART interrupts

	BlinkRed(2);
	BlinkGreen(2);
	
	Init_3Axis_Accelerometer(G15MODE);

	//Nokia_LCD_LED(ON);
	//Delaymsec(500);
	//Nokia_LCD_LED(OFF);
	
	signon();
	
	//Nokia_LCD_LED(ON);
	//Initialize_Nokia_LCD();			
	
	//Draw_Menu_Command();
	
	if(i2cmode==ACTIVE) Enumerate_TWI_Devices();
	
	printf("\n\r-");
	
	// Wait for commands on serial channel
	while (1) {
	
		// The log data function is started
		if(loggingmode==START) LogData(defaultlograte);
		
		// Turn logging off if button pressed again
		if((armpd2==YES)&&(!(PIND&(1<<PIND2)))&&(loggingmode==START)) {
		    loggingmode = HALT;
			printf("\n\rStop logging sensor data\n\r-");
			GREEN_LED(OFF);
			// wait until switch is released
			while(!(PIND&(1<<PIND2))) ;
			Delaymsec(200);
			}
		
		// If logging hasn't start see if button pressed
		if((armpd2==YES)&&(!(PIND&(1<<PIND2)))) {
		    loggingmode = START;
			printf("\n\rStart logging sensor data");
			Init_3Axis_Accelerometer(G15MODE);
			GREEN_LED(ON);
			if(FlashBlankCheck()==YES) {
				RED_LED(ON);
				FlashChipErase();
				RED_LED(OFF);
				}
			printf("\n\r-");
			bufferidptr = 1;
			pagecnt = 1;
			flashaddress = 0;
			// Halt until switch is released
			while(!(PIND&(1<<PIND2))) ;
			Delaymsec(200);
			}

		// Work starts here
		if((inchar = getcharnw())) {
		
		        if(0) ProcessLCDCommands(inchar);
				
			   	switch (inchar) {
					case 's': case 'S':
					     putchar(inchar);
						 signon();
						 if(i2cmode==ACTIVE)  Enumerate_TWI_Devices();
						 printf("\n\r-");
						 //printf("rxcnt=%d\n-",rxcnt); 
						 break;
					case 'm': case 'M':
						  putcharr(inchar);
						  MeasureAnalogInputs();
					      putcharr('-');
						  break;
					case 'd': case 'D':
						  putcharr(inchar);
						  DisplayMemory();
					      putcharr('-');
						  break;
					case 'c': case 'C':
						  putcharr(inchar);
						  DisplayCompass();
					      putcharr('-');
						  break;
					case 'p': case 'P':
							putcharr(inchar);
						    LoadMemory();
							putcharr('-');
							break;
					case 'a': case 'A':
						    DumpAnalogChs();
							xgravity = ygravity = 0;
							for(i=0; i<10; i++) {
								xgravity += AnalogCh(0,INTERNALREF);
								ygravity += AnalogCh(1,INTERNALREF);
								}
							xgrav = xgravity/10;
							ygrav = ygravity/10;
							printf(" Accelerometer x-axis=%3d\n",xgrav);
							printf(" Accelerometer y-axis=%3d\n",ygrav);
							// Reference battery voltage to internal 2.56v ref otherwise
							// input voltage value will not change as battery voltage drops
							printf(" Battery voltage=%3d\n",AnalogCh(2,INTERNALREF));
							printf(" Temperature sensor=%3d\n",AnalogCh(3,EXTERNALREF));
							printf(" Light sensor=%3d\n",AnalogCh(4,EXTERNALREF));
							putcharr('-');
							break;
					case 'v': case 'V':
						    ProgramVersion();
							putcharr('-');
							break;
					case ':':
						    ProgramMemory();
							//putcharr('-');
							break;
					case '1':
					        printf("l - write to SmartLCD continuously\n\r");
					        Write_Float_SmartLCD(SMARTLCDTWIADDRESS,'x',1.234);
							//while(!getcharnw()) {
							    //Enumerate_SmartLCD();
								//shortdelay(2000);
							//	}
							printf("\n\r-");
							break;
					case '2':
					        Execute_TWI_Command();
							break;
//					case 'c': case 'C':
//						    while(!(inchar=getcharnw()))  TestAVR();
//							signon();
//							break;
				    case 't': case 'T':
						    getsettime();
							printf("\n-");
							break;
					case 'g':
						    putcharr('g');
							Display_Gravity_UART(GONLY);
							break;
					case 'r':
						    putcharr('r');
						    numaxis = getchar();
							putchar(numaxis);
							Display_RateGyro_UART(numaxis);
							break;
					case 'e':  case 'E':
							putchar('e');
							FlashChipErase();
							printf("\n\r-");
							break;
					case 'l':  case 'L':
							putchar('l');
							LogDataToSerialFlash();
							printf("\n\r-");
							break;
					case '=':
							printf("rxcntmax=%d\n-",rxcntmax);
							rxcntmax = 0;
							printf("totalrxcnt=%d\n-",totalrxcnt);
							rxcntmax = 0;
							//if(frameerror==YES) puts("frame error\n\r");
							//else puts("no frame error\n\r");
							//if(dataoverrun==YES) puts("data overrun\n\r");
							//else puts("no data overrun\n\r");
							//dataoverrun = NO;
							//frameerror = NO;
							printf("xoncnt=%d\n",xoncnt);
							xoncnt = 0;
							xoffcnt = 0;
							break;
					case CR:
							if(flashprogrammingactive==NO)  printf("\n-");
							break;
					default:
							putcharr(inchar);
							//Delaymsec(20);
							break;
					}
				}

			   		 
		// Blink green led slowly to show AVR is alive
		if(mseccountdown1==0) {
			mseccountdown1 = loopsetting;
			if(loopmode)  {
			    RED_LED(ON);
				loopmode = 0;
				}
			else {
				RED_LED(OFF);
				loopmode = 1;
				}
			}
										
		// Clean up flash state machine if no new Intel
		// Hex records arrive for 3 seconds
		if((flashcountdown==0)&&(flashprogrammingactive==YES))
				CleanUpFlashState();
		}
}
